function m = gridnwc(d,k,w,n,sf,C,W,ks,t)

% function m = grid2w(d,k,n)
%     d -- k-space data
%     k -- k-trajectory, scaled -0.5 to 0.5
%     n -- image size

% preweight the data (really, unweight the data)
d = d.*w;

% convert to single column
d = d(:);
k = k(:);

% sampling factor
%sf = 2;
% kernel radius
kr = floor(W/2);
% kernel size (in samples)
%ks
G = 1;

% convert k-space samples to matrix indices
nx = ((n*sf)/2+1) + (n*sf)*real(k);
ny = ((n*sf)/2+1) + (n*sf)*imag(k);

% zero out output array
m = zeros((n*sf),(n*sf));

% loop over samples in kernel
for lx = -(kr):(kr),
  for ly = -(kr):(kr),

    % find nearest samples
    nxt = round(nx+lx);
    nyt = round(ny+ly);

    % 1D (taxicab) distance
	dx = abs(nx-nxt);
	dy = abs(ny-nyt);
	% 2D distance
	dxy = abs((nx-nxt)+i*(ny-nyt));
    % compute weighting for arbitrary kernel
	kwx = C(round((ks/2)+1+dx*(ks/(W/G))));
    kwy = C(round((ks/2)+1+dy*(ks/(W/G))));
	%kw =  C(round((ks/2)+1+dxy*(ks/(W/G))));

    % map samples outside the matrix to the edges
    nxt = max(nxt,1); nxt = min(nxt,(n*sf));
    nyt = max(nyt,1); nyt = min(nyt,(n*sf));
	
    % use sparse matrix to turn k-space trajectory into 2D matrix
    if(t==0)
		% cartesian-sym C(x,y)
	    m = m+sparse(nxt,nyt,d.*kwx'.*kwy',(n*sf),(n*sf));
    else
		% circular-sym C(x,y)
        m = m+sparse(nxt,nyt,d.*kw,(n*sf),(n*sf));
    end
  end;
end;

% zero out edge samples, since these may be due to samples outside
% the matrix
m(:,1) = 0; m(:,(n*sf)) = 0;
m(1,:) = 0; m((n*sf),:) = 0;


